//----------------------------------------------------------------------
// seqmemwrite
// version 1.0
// F. Abignoli
// 4/5/2013
//----------------------------------------------------------------------
// Initialisation de l'EEPROM pour le sequenceur
// Fabrique un nom pour chaque preset
// Initialise les donnees de base
// Programme prevu pour 24LC256
//----------------------------------------------------------------------


#include <Wire.h> //I2C library
#include <LiquidCrystal_I2C.h>
#if defined(ARDUINO) && ARDUINO >= 100
#define printByte(args)  write(args);
#else
#define printByte(args)  print(args,BYTE);
#endif

LiquidCrystal_I2C lcd(0x27,20,4);       // sLCD adress 0x27, 16 car, 2 lignes



#define MAXPROG 128
#define MAXBANK 2
#define PRESETSIZE  128
#define BANKSIZE  16384

char nompreset[17];   					// Nom du preset
byte SeqNotes[32];						// Sequence note des 2 pistes
byte SeqGates[32];						// Sequence gate (ou velocite) des 2 pistes
byte setup_Steps=16;					// Nombre de steps (<=16 si 2 pistes, <=32 si une piste)
byte setup_ModeTrack=0;					// Mode track : 0->2 pistes de 16, 1->1 piste de 32, 2->1 piste 16 avec velo	
byte setup_Bpm=120;						// BPM si horloge locale
byte setup_ClockIn=0;					// Type clock: 0->MIDI, 1:external analogique, 2:internal
byte setup_SensSeq=0;					// Sens de la sequence (up, down, etc...)
byte setup_ModeCV=0;     	     		// Mode CV actif (Octave, Hertz, SX150...)
byte setup_ModeOut=1;					// Type de sortie (0:MIDI, 1:CV, 2:MIDI+CV)
byte setup_ClockDivider=0;  			// Diviseur de l'horloge maitre( midi ou CV)




void eewrite_byte( int deviceaddress, unsigned int eeaddress, byte data ) 
{
int rdata = data;
  Wire.beginTransmission(deviceaddress);
  Wire.write((int)(eeaddress >> 8)); // MSB
  Wire.write((int)(eeaddress & 0xFF)); // LSB
  Wire.write(rdata);
  Wire.endTransmission();
  delay(5);
}

byte eeread_byte( int deviceaddress, unsigned int eeaddress ) {
  byte rdata = 0xFF;
  Wire.beginTransmission(deviceaddress);
  Wire.write((int)(eeaddress >> 8)); // MSB
  Wire.write((int)(eeaddress & 0xFF)); // LSB
  Wire.endTransmission();
  Wire.requestFrom(deviceaddress,1);
  if (Wire.available()) rdata = Wire.read();
    return rdata;
}

void setup() 
{
  //Wire.begin(); // initialise the connection
  lcd.init();             // init du lcd 
  lcd.backlight();        // Retroeclairage
  lcd.clear();            // Effacement de lecran
  lcd.setCursor(0,0);
}

void init_preset(byte mybank,byte mypreset)
{
int i;
int myaddress=0;
char bufnum[4];
int n;
    // Fabrication dun nom bidon au format
    // "PRESET xxx-Bx   "
    strcpy(nompreset,"PRESET ");
	
    if (mypreset+1<10) {
        strcat(nompreset,"00");
    }
    else {
      if (mypreset+1<100) {
        strcat(nompreset,"0");
      }
    }
    itoa(mypreset+1,bufnum,10);
    strcat(nompreset,bufnum);
    strcat(nompreset,"-B");
    
    itoa(mybank,bufnum,10);
    strcat(nompreset,bufnum);
    strcat(nompreset,"   ");
    // Initialisation de l'adresse du preset
    myaddress=(mybank*BANKSIZE)+(mypreset*PRESETSIZE);
	// Ecrit nom preset
    for (i=0;i<16;i++) {
      eewrite_byte( 0x50, myaddress++, nompreset[i]);
    }
	// Ecrit 32 steps de note
    for (i=0;i<32;i++) {
      eewrite_byte( 0x50, myaddress++, 36);
    }
	// Ecrit 32 steps de gate
    for (i=0;i<32;i++) {
      eewrite_byte( 0x50, myaddress++, 10);
    }
	setup_Steps=16;					// Nombre de steps (<=16 si 2 pistes, <=32 si une piste)
	setup_ModeTrack=0;					// Mode track : 0->2 pistes de 16, 1->1 piste de 32, 2->1 piste 16 avec velo	
	setup_Bpm=120;						// BPM si horloge locale
	setup_ClockIn=0;					// Type clock: 0->MIDI, 1:external analogique, 2:internal
	setup_SensSeq=0;					// Sens de la sequence (up, down, etc...)
	setup_ModeCV=0;     	     		// Mode CV actif (Octave, Hertz, SX150...)
	setup_ModeOut=1;					// Type de sortie (0:MIDI, 1:CV, 2:MIDI+CV)
	setup_ClockDivider=0;  			// Diviseur de l'horloge maitre( midi ou CV)
    // nombre de steps
    eewrite_byte( 0x50, myaddress++, setup_Steps);
	// Mode track
    eewrite_byte( 0x50, myaddress++, setup_ModeTrack);
	// BPM
    eewrite_byte( 0x50, myaddress++, setup_Bpm);
    // Type de clock
    eewrite_byte( 0x50, myaddress++, setup_ClockIn);
	// Sens du sequenceur
    eewrite_byte( 0x50, myaddress++, setup_SensSeq);
	// Mode CV
    eewrite_byte( 0x50, myaddress++, setup_ModeCV);
    // Type de sortie
    eewrite_byte( 0x50, myaddress++, setup_ModeOut);
	// Diviseur horloge
    eewrite_byte( 0x50, myaddress++, setup_ClockDivider);
	
}

void loop() 
{
int addr=0; // adresse de debut
int bank;   // Numero de bank
int i,j,k;  // variables de travail
	lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("Init. EEPROM...");
    lcd.setCursor(0,1);
    lcd.print("Bank....:");
    lcd.setCursor(0,2);
    lcd.print("Preset..:");
    for (i=0;i<MAXBANK;i++) {
        lcd.setCursor(10,1);
        lcd.print(i,DEC);
        for (j=0;j<MAXPROG;j++) {
          lcd.setCursor(10,2);
          lcd.print(j+1,DEC);
          lcd.print("   ");
          init_preset(i,j);
        }
    }
    lcd.clear();
    lcd.print("Fin de l'ecriture");
    while(1) {
    }
}